"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var util_1 = require("../util");
function default_1(compiler, next) {
    compiler.shims.push(util_1.wrap("process.__nexe=" + JSON.stringify({ resources: compiler.resources.index }) + ";"));
    compiler.shims.push(util_1.wrap("\"use strict\";\r\nObject.defineProperty(exports, \"__esModule\", { value: true });\r\nvar assert_1 = require(\"assert\");\r\nvar binary = process.__nexe;\r\nassert_1.ok(binary);\r\nvar manifest = binary.resources;\r\nvar directories = {};\r\nvar isString = function (x) { return typeof x === 'string' || x instanceof String; };\r\nvar isNotFile = function () { return false; };\r\nvar isNotDirectory = isNotFile;\r\nvar isFile = function () { return true; };\r\nvar noop = function () { };\r\nvar isDirectory = isFile;\r\nvar fs = require('fs');\r\nvar path = require('path');\r\nvar originalExistsSync = fs.existsSync;\r\nvar originalReadFile = fs.readFile;\r\nvar originalReadFileSync = fs.readFileSync;\r\nvar originalCreateReadStream = fs.createReadStream;\r\nvar originalReaddir = fs.readdir;\r\nvar originalReaddirSync = fs.readdirSync;\r\nvar originalStatSync = fs.statSync;\r\nvar originalStat = fs.stat;\r\nvar originalRealpath = fs.realpath;\r\nvar originalRealpathSync = fs.realpathSync;\r\nvar resourceStart = binary.layout.resourceStart;\r\nvar log = function (text) {\r\n    if ((process.env.DEBUG || '').toLowerCase().includes('nexe:require')) {\r\n        process.stderr.write('[nexe] - ' + text + '\\n');\r\n    }\r\n    else {\r\n        log = noop;\r\n    }\r\n};\r\nvar statTime = function () {\r\n    var stat = binary.layout.stat;\r\n    return {\r\n        dev: 0,\r\n        ino: 0,\r\n        nlink: 0,\r\n        rdev: 0,\r\n        uid: 123,\r\n        gid: 500,\r\n        blksize: 4096,\r\n        blocks: 0,\r\n        atime: new Date(stat.atime),\r\n        atimeMs: stat.atime.getTime(),\r\n        mtime: new Date(stat.mtime),\r\n        mtimeMs: stat.mtime.getTime(),\r\n        ctime: new Date(stat.ctime),\r\n        ctimMs: stat.ctime.getTime(),\r\n        birthtime: new Date(stat.birthtime),\r\n        birthtimeMs: stat.birthtime.getTime()\r\n    };\r\n};\r\nvar createStat = function (directoryExtensions, fileExtensions) {\r\n    if (!fileExtensions) {\r\n        return Object.assign({}, binary.layout.stat, directoryExtensions, { size: 0 }, statTime());\r\n    }\r\n    var size = directoryExtensions[1];\r\n    return Object.assign({}, binary.layout.stat, fileExtensions, { size: size }, statTime());\r\n};\r\nvar ownStat = function (filepath) {\r\n    var key = path.resolve(filepath);\r\n    if (directories[key]) {\r\n        return createStat({ isDirectory: isDirectory, isFile: isNotFile });\r\n    }\r\n    if (manifest[key]) {\r\n        return createStat(manifest[key], { isFile: isFile, isDirectory: isNotDirectory });\r\n    }\r\n};\r\nfunction makeLong(filepath) {\r\n    return path._makeLong && path._makeLong(filepath);\r\n}\r\nvar setupManifest = function () {\r\n    Object.keys(manifest).forEach(function (key) {\r\n        var entry = manifest[key];\r\n        var absolutePath = path.resolve(key);\r\n        var longPath = makeLong(absolutePath);\r\n        var normalizedPath = path.normalize(key);\r\n        if (!manifest[absolutePath]) {\r\n            manifest[absolutePath] = entry;\r\n        }\r\n        if (longPath && !manifest[longPath]) {\r\n            manifest[longPath] = entry;\r\n        }\r\n        if (!manifest[normalizedPath]) {\r\n            manifest[normalizedPath] = manifest[key];\r\n        }\r\n        var currentDir = path.dirname(absolutePath);\r\n        var prevDir = absolutePath;\r\n        while (currentDir !== prevDir) {\r\n            directories[currentDir] = directories[currentDir] || {};\r\n            directories[currentDir][path.basename(prevDir)] = true;\r\n            var longDir = makeLong(currentDir);\r\n            if (longDir && !directories[longDir]) {\r\n                directories[longDir] = directories[currentDir];\r\n            }\r\n            prevDir = currentDir;\r\n            currentDir = path.dirname(currentDir);\r\n        }\r\n    });\r\n    setupManifest = noop;\r\n};\r\n//naive patches intended to work for most use cases\r\nvar nfs = {\r\n    existsSync: function existsSync(filepath) {\r\n        var key = path.resolve(filepath);\r\n        if (manifest[key] || directories[key]) {\r\n            return true;\r\n        }\r\n        return originalExistsSync.apply(fs, arguments);\r\n    },\r\n    realpath: function realpath(filepath, options, cb) {\r\n        setupManifest();\r\n        if (isString(filepath) && manifest[filepath]) {\r\n            return process.nextTick(function () { return cb(null, filepath); });\r\n        }\r\n        return originalRealpath.call(fs, filepath, options, cb);\r\n    },\r\n    realpathSync: function realpathSync(filepath, options) {\r\n        setupManifest();\r\n        if (isString(filepath) && manifest[filepath]) {\r\n            return filepath;\r\n        }\r\n        return originalRealpathSync.call(fs, filepath, options);\r\n    },\r\n    readdir: function readdir(filepath, options, callback) {\r\n        setupManifest();\r\n        filepath = filepath.toString();\r\n        if ('function' === typeof options) {\r\n            callback = options;\r\n            options = { encoding: 'utf8' };\r\n        }\r\n        var dir = directories[path.resolve(filepath)];\r\n        if (dir) {\r\n            process.nextTick(function () {\r\n                //todo merge with original?\r\n                callback(null, Object.keys(dir));\r\n            });\r\n        }\r\n        else {\r\n            return originalReaddir.apply(fs, arguments);\r\n        }\r\n    },\r\n    readdirSync: function readdirSync(filepath, options) {\r\n        setupManifest();\r\n        filepath = filepath.toString();\r\n        var dir = directories[path.resolve(filepath)];\r\n        if (dir) {\r\n            return Object.keys(dir);\r\n        }\r\n        return originalReaddirSync.apply(fs, arguments);\r\n    },\r\n    readFile: function readFile(file, options, callback) {\r\n        setupManifest();\r\n        var entry = manifest[file] || manifest[path.resolve(file)];\r\n        if (!entry || !isString(file)) {\r\n            return originalReadFile.apply(fs, arguments);\r\n        }\r\n        var offset = entry[0], length = entry[1];\r\n        var resourceOffset = resourceStart + offset;\r\n        var encoding = isString(options) ? options : null;\r\n        callback = typeof options === 'function' ? options : callback;\r\n        fs.open(process.execPath, 'r', function (err, fd) {\r\n            if (err)\r\n                return callback(err, null);\r\n            fs.read(fd, Buffer.alloc(length), 0, length, resourceOffset, function (error, bytesRead, result) {\r\n                if (error) {\r\n                    return fs.close(fd, function () {\r\n                        callback(error, null);\r\n                    });\r\n                }\r\n                fs.close(fd, function (err) {\r\n                    if (err) {\r\n                        return callback(err, result);\r\n                    }\r\n                    callback(err, encoding ? result.toString(encoding) : result);\r\n                });\r\n            });\r\n        });\r\n    },\r\n    createReadStream: function createReadStream(file, options) {\r\n        setupManifest();\r\n        var entry = manifest[file] || manifest[path.resolve(file)];\r\n        if (!entry || !isString(file)) {\r\n            return originalCreateReadStream.apply(fs, arguments);\r\n        }\r\n        var offset = entry[0], length = entry[1];\r\n        var resourceOffset = resourceStart + offset;\r\n        var opts = !options ? {} : isString(options) ? { encoding: options } : options;\r\n        return fs.createReadStream(process.execPath, Object.assign({}, opts, {\r\n            start: resourceOffset,\r\n            end: resourceOffset + length\r\n        }));\r\n    },\r\n    readFileSync: function readFileSync(file, options) {\r\n        setupManifest();\r\n        var entry = manifest[file] || manifest[path.resolve(file)];\r\n        if (!entry || !isString(file)) {\r\n            return originalReadFileSync.apply(fs, arguments);\r\n        }\r\n        var offset = entry[0], length = entry[1];\r\n        var resourceOffset = resourceStart + offset;\r\n        var encoding = isString(options) ? options : null;\r\n        var fd = fs.openSync(process.execPath, 'r');\r\n        var result = Buffer.alloc(length);\r\n        fs.readSync(fd, result, 0, length, resourceOffset);\r\n        fs.closeSync(fd);\r\n        return encoding ? result.toString(encoding) : result;\r\n    },\r\n    statSync: function statSync(filepath) {\r\n        var stat = isString(filepath) && ownStat(filepath);\r\n        if (stat) {\r\n            return stat;\r\n        }\r\n        return originalStatSync.apply(fs, arguments);\r\n    },\r\n    stat: function stat(filepath, callback) {\r\n        var stat = isString(filepath) && ownStat(filepath);\r\n        if (stat) {\r\n            process.nextTick(function () {\r\n                callback(null, stat);\r\n            });\r\n        }\r\n        else {\r\n            return originalStat.apply(fs, arguments);\r\n        }\r\n    }\r\n};\r\nif (typeof fs.exists === 'function') {\r\n    nfs.exists = function (filepath, cb) {\r\n        cb = cb || noop;\r\n        var exists = nfs.existsSync(filepath);\r\n        process.nextTick(function () { return cb(exists); });\r\n    };\r\n}\r\nObject.assign(fs, nfs);\r\nvar patches = process.nexe.patches;\r\ndelete process.nexe;\r\npatches.internalModuleReadFile = function (original) {\r\n    var args = [];\r\n    for (var _i = 1; _i < arguments.length; _i++) {\r\n        args[_i - 1] = arguments[_i];\r\n    }\r\n    var filepath = args[0];\r\n    setupManifest();\r\n    if (manifest[filepath]) {\r\n        log('read     (hit)              ' + filepath);\r\n        return nfs.readFileSync(filepath, 'utf-8');\r\n    }\r\n    log('read          (miss)       ' + filepath);\r\n    return original.call.apply(original, [this].concat(args));\r\n};\r\npatches.internalModuleStat = function (original) {\r\n    var args = [];\r\n    for (var _i = 1; _i < arguments.length; _i++) {\r\n        args[_i - 1] = arguments[_i];\r\n    }\r\n    setupManifest();\r\n    var filepath = args[0];\r\n    if (manifest[filepath]) {\r\n        log('stat     (hit)              ' + filepath + '   ' + 0);\r\n        return 0;\r\n    }\r\n    if (directories[filepath]) {\r\n        log('stat dir (hit)              ' + filepath + '   ' + 1);\r\n        return 1;\r\n    }\r\n    var res = original.call.apply(original, [this].concat(args));\r\n    if (res === 0) {\r\n        log('stat          (miss)        ' + filepath + '   ' + res);\r\n    }\r\n    else if (res === 1) {\r\n        log('stat dir      (miss)        ' + filepath + '   ' + res);\r\n    }\r\n    else {\r\n        log('stat                 (fail) ' + filepath + '   ' + res);\r\n    }\r\n    return res;\r\n};\r\n"));
    compiler.shims.push(util_1.wrap("\n    if (process.argv[1] && process.env.NODE_UNIQUE_ID) {\n      const cluster = require('cluster')\n      cluster._setupWorker()\n      delete process.env.NODE_UNIQUE_ID\n    }\n  "));
    if (compiler.options.fakeArgv !== false) {
        var nty = !process.stdin.isTTY;
        var input = nty
            ? '[stdin]'
            : JSON.stringify(path_1.relative(compiler.options.cwd, compiler.options.input));
        compiler.shims.push(util_1.wrap("\n        var r = require('path').resolve; \n        process.argv.splice(1,0, " + (nty ? "'" + input + "'" : "r(" + input + ")") + ");"));
    }
    return next();
}
exports.default = default_1;
